# -*- coding: utf-8 -*-

"""
    abouttag.test

    Run tests for the whole abouttag module.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unittest
from about import *
from abouttag.fluiddb import *
from abouttag.uri import *
from abouttag.database import *
from abouttag.objects import *
from abouttag.location import *
from abouttag.books import *
from abouttag.music import *
from abouttag.film import *

unittest.main()



