from amazon import get_about_tag_for_item

print get_about_tag_for_item('http://www.amazon.com/Renbourn-Stefan-Grossman-Live-Concert/dp/B000000E8R/ref=sr_1_7?ie=UTF8&qid=1308066155&sr=8-7')

print get_about_tag_for_item('http://www.amazon.com/gp/product/B0027J9SM8/ref=dm_mu_dp_trk1')


